"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.plugin = void 0;
const graphql_1 = require("graphql");
const plugin_helpers_1 = require("@graphql-codegen/plugin-helpers");
const custom_type_class_js_1 = require("./custom-type-class.js");
const file_type_js_1 = require("./file-type.js");
const input_type_visitor_js_1 = require("./input-type-visitor.js");
const operation_visitor_js_1 = require("./operation-visitor.js");
const plugin = (schema, documents, config) => {
    const allAst = (0, graphql_1.concatAST)(documents.map(v => v.document));
    const allFragments = [
        ...allAst.definitions.filter(d => d.kind === graphql_1.Kind.FRAGMENT_DEFINITION).map(fragmentDef => ({
            node: fragmentDef,
            name: fragmentDef.name.value,
            onType: fragmentDef.typeCondition.name.value,
            isExternal: false,
        })),
        ...(config.externalFragments || []),
    ];
    let visitor;
    switch (config.fileType) {
        case file_type_js_1.FileType.FRAGMENT:
        case file_type_js_1.FileType.OPERATION: {
            visitor = new operation_visitor_js_1.OperationVisitor(schema, config, allFragments);
            break;
        }
        case file_type_js_1.FileType.INPUT_TYPE: {
            visitor = new input_type_visitor_js_1.InputTypeVisitor(schema, config);
            break;
        }
        case file_type_js_1.FileType.CUSTOM_TYPES: {
            visitor = new custom_type_class_js_1.CustomTypeClassVisitor(schema, config);
            break;
        }
    }
    if (!visitor) {
        return { content: '' };
    }
    const visitResult = (0, plugin_helpers_1.oldVisit)(allAst, visitor);
    const additionalContent = visitor.additionalContent();
    const imports = visitor.getImports();
    return {
        prepend: [`package ${visitor.getPackage()};\n`, ...imports],
        content: '\n' +
            [...visitResult.definitions.filter(a => a && typeof a === 'string'), additionalContent].join('\n'),
    };
};
exports.plugin = plugin;
